# Copyright (C) 2017-2019  Lars Wirzenius
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import ick2


class BuildsAPI(ick2.ResourceApiBase):  # pragma: no cover

    def __init__(self, state):
        super().__init__('builds', state)

    def get_routes(self, path):
        return [
            {
                'method': 'POST',
                'path': path,
                'callback': self.POST(self.create),
            },
            {
                'method': 'GET',
                'path': path,
                'callback': self.GET(self.list),
            },
            {
                'method': 'GET',
                'path': '{}/<name:path>'.format(path),
                'callback': self.GET(self.show),
            },
            {
                'method': 'PUT',
                'path': '{}/<name:path>'.format(path),
                'callback': self.PUT(self.update),
            },
            {
                'method': 'DELETE',
                'path': '{}/<name:path>'.format(path),
                'callback': self.DELETE(self.delete),
            },
        ]

    def get_resource_name(self, resource):
        return resource['build']

    def create(self, body, **kwargs):  # pragma: no cover
        raise ick2.MethodNotAllowed('Creating builds directly is not allowed')

    def update(self, body, name, **kwargs):  # pragma: no cover
        raise ick2.MethodNotAllowed('Updating builds directly is not allowed')

    def list(self, **kwargs):
        result = super().list(**kwargs)
        items = result[self._type_name]
        items.sort(key=lambda x: x.get('build_number'))
        result[self._type_name] = items
        return result
